from _py3coap import UriNotFoundError, HandshakeError, MethodNotAllowedError, ReadTimeoutError, WriteTimeoutError

class _baseException(Exception):
    def __init__(self, *args, **kwargs):
        default_message = ""

        # if no arguments are passed set the first positional argument
        # to be the default message. To do that, we have to replace the
        # 'args' tuple with another one, that will only contain the message.
        # (we cannot do an assignment since tuples are immutable)
        if not (args or kwargs):
            args = (default_message,)

        # Call super constructor
        super().__init__(*args, **kwargs)


class MalformedUri(_baseException):
    pass


class MissingCredentials(_baseException):
    def __init__(self, *args, **kwargs):
        default_message = "Missing required credentials"

        if not (args or kwargs):
            args = (default_message,)

        # Call super constructor
        super().__init__(*args, **kwargs)



import _py3coap
from .errors import MalformedUri, MissingCredentials

GET = 0
PUT = 1
POST = 2


def setDebugLevel(level):
    _py3coap.DebugLevel(level)


def Request(uri, payload=None, method=GET, ident=None, key=None):

    protocol = uri.split("//")

    if not (protocol[0] == "coap:" or protocol[0] == "coaps:"):
        raise MalformedUri("Missing protocol")

    uri = protocol[1].split("/")

    if len(uri) < 2:
        raise MalformedUri(uri)

    host = uri[0]

    if not ":" in host:
        raise MalformedUri("Missing port")

    dest = "/".join(uri[1:])

    if protocol[0] == "coap:":
        if payload is None:
            return _py3coap.Request(host, dest)
        else:
            method = PUT if method == GET else method
            if method == PUT:
                return _py3coap.PutRequest(host, dest, payload)
            elif method == POST:
                return _py3coap.PostRequest(host, dest, payload)

    elif protocol[0] == "coaps:":
        if ident is None or key is None:
            raise MissingCredentials

        if payload is None:
            return _py3coap.DTLSRequest(host, dest, ident, key)
        else:
            method = PUT if method == GET else method

            if method == PUT:
                result = _py3coap.DTLSPutRequest(host, dest, payload, ident, key)
                return result
            elif method == POST:
                result = _py3coap.DTLSPostRequest(host, dest, payload, ident, key)
                return result

    else:
        raise MalformedUri("Unknow protocol: {}".format(protocol[0]))

    return None



from setuptools import Extension
from setuptools import setup


setup(
    name="py3coap",
    description="A GOLANG python extension for COAP/COAPS-requests",
    url="https://github.com/moroen/pycoap",
    version="0.7.1",
    author="moroen",
    author_email="no@email.com",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: Implementation :: CPython",
    ],
    ext_modules=[Extension("_py3coap", ["src/py3coap/py3coap.go"])],
    data_files=[('src/py3coap', ['src/py3coap/py3coap.h', 'src/py3coap/py3coap.c'])],
    packages=["py3coap"],
    build_golang={"root": "github.com/moroen/py3coap"},
    setup_requires=["setuptools-golang"],
    scripts=["pycoap-client"],
)
